@@ -19,12 +19,15 @@
 import os
 from pathlib import Path
 
+import crypt4gh.keys
 import requests
 
 from ghga_connector.core import exceptions
 from ghga_connector.core.api_calls import (
     UploadStatus,
     await_download_url,
+    get_download_urls,
+    get_file_header_envelope,
     get_part_upload_urls,
     patch_multipart_upload,
     start_multipart_upload,
@@ -105,7 +108,7 @@ def upload(  # noqa C901, pylint: disable=too-many-statements,too-many-branches
         message_display.failure(f"The file with id '{file_id}' was already uploaded.")
         raise error
     except exceptions.RequestFailedError as error:
-        message_display.failure("The request has failed.")
+        message_display.failure("The request to start a multipart upload has failed.")
         raise error
 
     try:
@@ -175,36 +178,71 @@ def download(  # pylint: disable=too-many-arguments
         message_display.failure(f"The directory {output_dir} does not exist.")
         raise exceptions.DirectoryDoesNotExistError(output_dir=output_dir)
 
+    if not os.path.isfile(pubkey_path):
+        message_display.failure(f"The file {pubkey_path} does not exist.")
+        raise exceptions.PubKeyFileDoesNotExistError(pubkey_path=pubkey_path)
+
     if not check_url(api_url):
         message_display.failure(f"The url {api_url} is currently not reachable.")
         raise exceptions.ApiNotReachableError(api_url=api_url)
 
-    download_url, file_size = await_download_url(
-        api_url=api_url,
-        file_id=file_id,
-        max_wait_time=max_wait_time,
-        message_display=message_display,
-        pubkey_path=pubkey_path,
-    )
-
-    # perform the download:
+    public_key = crypt4gh.keys.get_public_key(pubkey_path)
 
+    # check output file
     output_file = os.path.join(output_dir, file_id)
     if os.path.isfile(output_file):
         message_display.failure(f"The file {output_file} already exists.")
         raise exceptions.FileAlreadyExistsError(output_file=output_file)
 
+    # stage download and get file size
+    download_url_tuple = await_download_url(
+        api_url=api_url,
+        file_id=file_id,
+        max_wait_time=max_wait_time,
+        message_display=message_display,
+    )
+
+    # get file header envelope
+    try:
+        envelope = get_file_header_envelope(
+            file_id=file_id,
+            api_url=api_url,
+            public_key=public_key,
+        )
+    except (
+        exceptions.FileNotRegisteredError,
+        exceptions.EnvelopeNotFoundError,
+        exceptions.ExternalApiError,
+    ) as error:
+        message_display.failure(
+            f"The request to get an envelope for file {file_id} failed."
+        )
+        raise error
+
+    # put envelope in file
+    with open(output_file, "wb") as file:
+        file.write(envelope)
+
+    # perform the download
     try:
         download_parts(
-            file_size=file_size,
-            download_url=download_url,
+            file_id=file_id,
+            api_url=api_url,
             output_file=output_file,
             part_size=part_size,
+            file_size=download_url_tuple[1],
         )
     except exceptions.MaxRetriesReachedError as error:
         # Remove file, if the download failed.
         os.remove(output_file)
         raise error
+    except exceptions.NoS3AccessMethodError as error:
+        message_display.failure(
+            f"The request to return information for file {file_id}"
+            + " did not return an S3 access method."
+        )
+        os.remove(output_file)
+        raise error
 
     message_display.success(
         f"File with id '{file_id}' has been successfully downloaded."
@@ -213,20 +251,27 @@ def download(  # pylint: disable=too-many-arguments
 
 def download_parts(
     *,
-    file_size: int,
-    download_url: str,
+    file_id: str,
+    api_url: str,
     output_file: str,
     part_size: int,
+    file_size: int,
 ) -> None:
     """
     Downloads a file using a specific download_url to download all its parts.
     """
 
-    file_parts = download_file_parts(
-        download_url=download_url,
-        part_size=part_size,
-        total_file_size=file_size,
+    download_urls = get_download_urls(
+        api_url=api_url,
+        file_id=file_id,
     )
-    with open(output_file, "wb") as file:
-        for part in file_parts:
-            file.write(part)
+    file_parts = download_file_parts(
+        download_urls=download_urls, part_size=part_size, total_file_size=file_size
+    )
+
+    with open(output_file, "ab") as file:
+        try:
+            for file_part in file_parts:
+                file.write(file_part)
+        except exceptions.NoS3AccessMethodError as error:
+            raise error
